# GStreamer test on Variscite IMX-based platforms

## Portability based on Variscite Debian distribution and Docker 
The objective of this tutorial is to show the application portability capabilities enabled 
by the Variscite Debian distribution and Docker. A GStreamer test is considered 
as a reference application to be executed both on a development environment (x86 platform) 
and on the target device (Variscite Development Kit). The devices and the graphic server are 
abstracted as resources to be passed to a Docker image in which the application is deployed. 


## Prerequisites
The following components are necessary to test the application:
* Variscite Development Kit in [1](#references).
* Variscite Debian or Yocto BSP [2](#references).
* Docker. Installation instructions in [3](#references).

## The reference application
As a reference application, the GStreamer gst-launch-1.0 tool is used to build a Gstreamer pipeline
in which a test video data is produced by using videotestsrc [4](#references). The videotestsrc 
element can be used to produce test video data in a wide variety of formats while controlling the 
video flow through available "pattern" properties. The gst-launch-1.0 has been selected as a reference
application since it is publicly available and accepts options that are common to all GStreamer 
applications.

## Executing the application on x86 platforms using X.org
The application can be easily used to build and test a GStreamer pipeline in x86 platforms running 
a Linux distribution based on X.org. A simple pipeline can be created with the following command:
```
$ gst-launch-1.0 videotestsrc ! autovideosink
``` 
The GStreamer test appears on the screen by using the X.org graphic server.

## Executing the application on IMX platforms using Wayland
The application can be easily ported in the platform by creating a Docker image through the provided 
Dockerfile:
```
FROM debian:buster-slim

ENV DEBIAN_FRONTEND noninteractive

RUN apt-get -y update && apt-get install -y --no-install-recommends \
    libgstreamer1.0-0 \
    gstreamer1.0-plugins-base \
    gstreamer1.0-plugins-good \
    gstreamer1.0-plugins-bad \
    gstreamer1.0-plugins-ugly \
    gstreamer1.0-libav \
    gstreamer1.0-doc \
    gstreamer1.0-tools \
    gstreamer1.0-x \
    gstreamer1.0-alsa \
    gstreamer1.0-gl \
    gstreamer1.0-gtk3 \
    gstreamer1.0-pulseaudio \
    && apt-get clean && apt-get autoremove && rm -rf /var/lib/apt/lists/*
	
ENTRYPOINT ["gst-launch-1.0"] 

CMD ["videotestsrc", "!", "waylandsink", "sync=false"]
``` 
In the Dockerfile the reference buster-slim image is used as a starting point, then gstreamer1.0 and 
all dependencies necessary to execute GStreamer applications are installed. The gst-launch-1.0 is 
defined as an entrypoint to be executed when the image is running. In the CMD instruction, 
the pipeline is created by using a video test source and a waylandsink property.

The new Docker image supporting GStreamer and running the application can be created by executing the 
following command in the directory where is located the Dockerfile:
```
$ docker build -t gstreamer-test .
``` 
Once the image is created the GStreamer application can be executed with: 
```
$ docker run -v /run/user/:/run/user/ -e XDG_RUNTIME_DIR=$XDG_RUNTIME_DIR -t gstreamer-test
``` 
The GStreamer test will appear on the board display by using the Wayland graphic server.

## References:
1. https://www.variscite.com/products/evaluation-kits/
2. http://variwiki.com/
3. https://docs.docker.com/install/linux/docker-ce/debian/
4. https://gstreamer.freedesktop.org/documentation/videotestsrc/
